@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -22,6 +22,7 @@ import datetime
 import decimal
 import fractions
 import functools
+import inspect
 import io
 import numbers
 import uuid
@@ -61,9 +62,24 @@ def try_issubclass(thing, superclass):
         return False
 
 
+def is_a_new_type(thing):
+    # At runtime, `typing.NewType` returns an identity function rather
+    # than an actual type, but we can check whether that thing matches.
+    return (
+        hasattr(thing, "__supertype__")
+        and hasattr(typing, "NewType")
+        and inspect.isfunction(thing)
+        and getattr(thing, "__module__", None) == "typing"
+    )
+
+
 def is_a_type(thing):
     """Return True if thing is a type or a generic type like thing."""
-    return isinstance(thing, type) or isinstance(thing, typing_root_type)
+    return (
+        isinstance(thing, type)
+        or isinstance(thing, typing_root_type)
+        or is_a_new_type(thing)
+    )
 
 
 def from_typing_type(thing):
@@ -161,6 +177,7 @@ _global_type_lookup = {
     set: st.builds(set),
     frozenset: st.builds(frozenset),
     dict: st.builds(dict),
+    type(lambda: None): st.functions(),
     # Built-in types
     type(Ellipsis): st.just(Ellipsis),
     type(NotImplemented): st.just(NotImplemented),
@@ -320,3 +337,15 @@ else:
     @register(typing.Iterator, st.iterables(st.nothing()))
     def resolve_Iterator(thing):
         return st.iterables(st.from_type(thing.__args__[0]))
+
+    @register(typing.Callable, st.functions())
+    def resolve_Callable(thing):
+        # Generated functions either accept no arguments, or arbitrary arguments.
+        # This is looser than ideal, but anything tighter would generally break
+        # use of keyword arguments and we'd rather not force positional-only.
+        if not thing.__args__:  # pragma: no cover  # varies by minor version
+            return st.functions()
+        return st.functions(
+            like=(lambda: None) if len(thing.__args__) == 1 else (lambda *a, **k: None),
+            returns=st.from_type(thing.__args__[-1]),
+        )
